import pyglet
from pyglet.window import key, mouse
import grfx
import squirtle
import isomap
from stats import stats
from camera import Camera
import Box2D.Box2D as b2
from blob import Blob
import scene
import physics
import time
import animation
import math



class MainScene(scene.Scene):
    def __init__(self, name, win):
        scene.Scene.__init__(self, name)
        
        self.camera = Camera()
        self.background = squirtle.SVG('data/sky.svg', invert_y = True)
        self.world_proto = squirtle.SVG('data/world_proto.svg', invert_y = True)
        
        self.setup_physics()
        self.edges = physics.edges_from_svg(self.world_proto, self.world)
        
        self.isomap = isomap.IsoGrid(self)

        self.avatar = Blob(30, 50, 15, self.world)
        
        self.blobs = [self.avatar,
                      Blob(110, 30, 5, self.world),
                      Blob(130, 30, 5, self.world), 
                      Blob(170, 30, 5, self.world),
                      Blob(150, 40, 5, self.world),
                      Blob(170, 40, 5, self.world)]

        self.keyboard = key.KeyStateHandler()
        win.push_handlers(self.keyboard)

        
        self.creatures = [animation.Load('data/rabbit.skel')]

    def setup_physics(self):
        self.worldAABB = b2.b2AABB()
        self.worldAABB.lowerBound = (-8192*physics.SCALE, -768*physics.SCALE)
        self.worldAABB.upperBound = (8192*physics.SCALE, 2000*physics.SCALE)
        gravity = (0.0, -20.0)
        
        doSleep = True
        #self.contact_listener = ContactListener()
        self.world = b2.b2World(self.worldAABB, gravity, doSleep)
        #self.world.SetContactListener(self.contact_listener)

    def on_key_press(self, symbol, modifiers):
        pass

    def on_mouse_press(self, x, y, button, modifiers):
        self.click_start = time.time()
    
    def on_mouse_release(self, x, y, button, modifiers):
        x += self.camera.x
        y += self.camera.y
        scale = time.time() - self.click_start + 1
        blob = Blob(x*physics.SCALE, y*physics.SCALE, scale*10, self.world)
        self.blobs.append(blob)

    def update(self, dt):

        if self.keyboard[key.LEFT]:
            self.avatar.phys_obj.body.ApplyForce(b2.b2Vec2(-100, 0), self.avatar.phys_obj.body.position)
            #self.avatar.phys_obj.body.velocity = b2.b2Vec2(-100, 0)

        if self.keyboard[key.RIGHT]:
            self.avatar.phys_obj.body.ApplyForce(b2.b2Vec2(100, 0), self.avatar.phys_obj.body.position)
            #self.avatar.phys_obj.body.velocity = b2.b2Vec2(100, 0)

        if self.avatar.x/physics.SCALE > self.camera.x + 1024:
            self.camera.x += 1024
        if self.avatar.x/physics.SCALE < self.camera.x:
            self.camera.x -= 1024

        for b in self.blobs:
            b.update(dt)
        self.isomap.update(dt, self.camera, 1/physics.SCALE)

        for x in self.blobs[1:]:
            x.phys_obj.body.ApplyTorque(10 * x.r * math.sin(time.time()))
        
        if dt > .1: dt = .1
        self.world.Step(dt, 10, 10)
        self.world.Validate()

    def draw(self):
        gl = pyglet.gl
        gl.glColor4f(1,1,1,1)       
        self.background.draw(0,0)

        gl.glPushMatrix()
        gl.glTranslatef(-self.camera.x, -self.camera.y, 0)
        self.world_proto.draw(0,0)

        def draw_world():
            gl.glPushMatrix()
            gl.glScalef(physics.SCALE, physics.SCALE, 0)
            self.world_proto.draw(0,0)
            gl.glPopMatrix()

        #setup isomap
        self.isomap.draw_commands =  [draw_world] +  [b.delayed_draw() for b in self.blobs]

        
        gl.glPopMatrix()

        #draw render texture
        self.isomap.isoshader.use()
        texture = self.isomap.fbo.texture
        grfx.draw_image(0, 0, texture.width, texture.height, texture)
        self.isomap.isoshader.stop()

        
